@@ -9,7 +9,7 @@ use vars qw(@ISA $VERSION);
 
 @ISA = qw(HTML::CalendarMonth::DateTool);
 
-$VERSION = '0.01';
+$VERSION = '0.02';
 
 use DateTime;
 
@@ -42,7 +42,7 @@ sub dow {
 
 sub add_days {
   my($self, $delta, $day, $month, $year) = @_;
-  $delta || croak "Delta (in days) required.\n";
+  defined $delta || croak "Delta (in days) required.\n";
   $day   || croak "Day required.\n";
   $month ||= $self->month;
   $year  ||= $self->year;
@@ -57,8 +57,8 @@ sub week_of_year {
   $month ||= $self->month;
   $year  ||= $self->year;
   my $dt = $self->_new_dt($year, $month, $day);
-  my($year, $week) = $dt->week;
-  ($year, $week);
+  # returns ($year, $week)
+  $dt->week;
 }
 
 sub _new_dt {
